# -*- coding: utf-8 -*-

# chirptext's package version information
__author__ = "Le Tuan Anh"
__email__ = "tuananh.le@ntu.edu.sg"
__copyright__ = "Copyright (c) 2020, BLIP lab, NTU, Singapore"
__credits__ = []
__license__ = "GNU General Public License v3 (GPLv3)"
__description__ = "Language Experience Overview (LEO) Report Generator"
__url__ = "https://blipntu.github.io/LEO/"
__maintainer__ = "Le Tuan Anh"
__version_major__ = "0.1"
__version__ = "{}a5.post1".format(__version_major__)
__version_long__ = "{} - Alpha".format(__version_major__)
__status__ = "3 - Alpha"
